/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Dictionary module.
 * 
 * This file contains all dictionary-related stuff internal to waaad daemon.
 * 
 * Note that outside the dictionary itself, the functions defined in dictionary-api.h 
 * should be sufficient for most uses of the dictionary.
 */

#ifndef _DICTIONARY_H
#define _DICTIONARY_H

/* Include the API for the dictionary */
#include <waaad/dictionary-api.h>

/* Include definitions for hard-coded values */
#include "dict-hardcoded.h"

/* Need also the definition of the uti_list_t type */
#include "utils.h"

/* Functions called only in the daemon */

/*
 * FUNCTION:	dict_init
 *
 * PARAMETERS:
 *  -
 *
 * DESCRIPTION: 
 *  Initialize the dictionary facility. 
 *
 * RETURN VALUE:
 *   0	: Application is now ready to use the dictionary.
 *  !0  : An error occurred. 
 */
int dict_init ( void );

/*
 * FUNCTION:	dict_fini
 *
 * PARAMETERS:
 *  -
 *
 * DESCRIPTION: 
 *  Terminates the dictionary facility. No dict_* function must be called after this one.
 *
 * RETURN VALUE:
 *   0 : dict facility closed properly.
 *  !0 : an error occurred (we may ignore it)
 */
int dict_fini ( void );

/*
 * FUNCTION:	dict_iterate_rules
 *
 * PARAMETERS:
 *   parent : A command or AVP dictionary object.
 *   data   : a pointer that will be passed to all callback calls. The content is opaque to the dictionary.
 *   cb     : a function to be called for each rule belonging to the parent object.
 *
 * DESCRIPTION: 
 *  Allows a callback to be called on each rule defined in the dictionary for the parent object.
 * A data pointer can be passed to the function, and it will be passed to the cb function in turn.
 * The cb function must return 0 when it encounters no error. If a non-0 value is returned, the iteration stops
 * and the returned value is returned by this function.
 * IMPORTANT: the cb function MUST NOT call the "dict_new" function. It would dead-lock.
 * The cb function is called with the data parameter, and a pointer to the rule data as arguments.
 * The order of the rules with which the cb function is called is not related to the position of the AVPs in the ruleset.
 * i.e. the cb function may be called with an "optional" rule before being called with a "fixed head" rule.
 *
 * RETURN VALUE:
 *   0     : the iteration occurred successfully.
 *   EINVAL: a parameter is invalid.
 *   !0    : an error code returned by the cb function.
 */
int dict_iterate_rules ( dict_object_t *parent, void * data, int (*cb)(void *, dict_rule_data_t *) );

/* Retrieve the list of vendors defined in the dictionary (to be used as Supported-Vendor-Id AVP in CER / CEA 
 *
 * The assumption is that the vendor was defined in the dictionary only if at least some of its AVPs are supported, otherwise it's not useful...
 */
uint32_t * dict_get_vendorid_list();

/* Retrieve the location of the dispatch callback list for any object. The content of this list is transparent to the dictionary module */
int dict_disp_cb(dict_object_type_t type, dict_object_t *dict, uti_list_t ** cb_list);

/* The following array can be used to retrieve the name of a base datatype */
extern char * type_base_name[];


/* The following object refers to the command object used for rules when the 'E' bit is set */
dict_object_t	* dict_cmd_error;


/* The following may be useful for debug */
#define DUMP_AVPFL_str	"%c%c"
#define DUMP_AVPFL_val(_val) (_val & AVP_FLAG_VENDOR)?'V':'-' , (_val & AVP_FLAG_MANDATORY)?'M':'-'
#define DUMP_CMDFL_str	"%c%c%c%c"
#define DUMP_CMDFL_val(_val) (_val & CMD_FLAG_REQUEST)?'R':'-' , (_val & CMD_FLAG_PROXIABLE)?'P':'-' , (_val & CMD_FLAG_ERROR)?'E':'-' , (_val & CMD_FLAG_RETRANSMIT)?'T':'-'
void dump_dictionary(void);
void dict_dump( dict_object_t * object, int parent, int depth);


#endif /* ! _DICTIONARY_H */

